import { COREN_DAYS_VALUES, ISO_TO_COREN_DAYS_VALUES } from '../constants/date';
import { IWeekDaysValues } from '../interfaces/IWeekDaysValues';

export const seconds_in_milliseconds = (seconds: number) => seconds * 1000;
export const minutes_in_milliseconds = (minutes: number) => seconds_in_milliseconds(60 * minutes);
export const hours_in_milliseconds = (hours: number) => minutes_in_milliseconds(60 * hours);
export const days_in_milliseconds = (days: number) => hours_in_milliseconds(24 * days);

export function getDateOnMidnight(date: Date): Date {
  const midnight = new Date(date);
  midnight.setUTCHours(0,0,0,0)
  return midnight;
}

export function getDateAtEndOfDay(date: Date): Date {
  const midnight = new Date(date);
  midnight.setUTCHours(23,59,59,999)
  return midnight;
}

export function getDateDaysAgo(date: Date, days: number): Date {
  const timestamp = getDateOnMidnight(date).getTime();

  return new Date(timestamp - days_in_milliseconds(days));
}

export function getDayOfTheSameWeek(date: Date, day: IWeekDaysValues): Date {
  const weekday = ISO_TO_COREN_DAYS_VALUES[date.getUTCDay()]
  if (day === weekday) return date;

  return getDateDaysAgo(date, weekday - day);
}

export function isSameWeek(date: Date, another_date: Date): boolean {
  const init = getDayOfTheSameWeek(date, COREN_DAYS_VALUES.MONDAY).getTime();
  const end = getDateAtEndOfDay(getDayOfTheSameWeek(date, COREN_DAYS_VALUES.SUNDAY)).getTime();
  
  const compare = another_date.getTime();

  return init <= compare && compare <= end;
}

