import CODE_MESSAGES from '../constants/codeMessages';
import { COMPARABLE_TYPES_AND_OPERATORS } from '../constants/comparableAndOperators';
import Exceptions from '../exceptions/exceptions';
import ICompare from '../interfaces/ICompare';
import IOperator from '../interfaces/IOperators';
import DateString from './date_string';

class DateComparatorHelper {
  public type_of: ICompare;

  public primary_value_to_compare: unknown;

  public secondary_value_to_compare: unknown;

  private value: Date;

  private operator: IOperator;

  private value_to_compare: unknown;

  constructor(operator: IOperator, value: Date, value_to_compare: unknown) {
    this.operator = operator;
    this.value = value;
    this.value_to_compare = value_to_compare;
    this.checkDateType();
  }

  private checkDateType(): ICompare {
    if (COMPARABLE_TYPES_AND_OPERATORS.date.includes(this.operator)) {
      this.type_of = 'date';
      this.primary_value_to_compare = this.value;
      this.setJourneyValueToCompare();
      return this.type_of;
    }

    if (COMPARABLE_TYPES_AND_OPERATORS.datetime.includes(this.operator)) {
      this.type_of = 'datetime';
      this.primary_value_to_compare = this.returnDateTimeValue(this.value);
      this.setJourneyValueToCompare();
    }

    return this.type_of;
  }

  private setJourneyValueToCompare(): void {
    let value: unknown;
    switch (typeof this.value_to_compare) {
      case 'number':
        value = this.value_to_compare;
        break;

      case 'string':
        value = this.stringEventValueToCompare();
        break;

      default:
        break;
    }

    this.secondary_value_to_compare = value;
  }

  private stringEventValueToCompare(): unknown {
    const date_string = new DateString(this.value_to_compare as string);
    if (!date_string.is_date) throw new Exceptions.BusinessError(CODE_MESSAGES.INVALID_TRIGGER_EVENT_VALUE);

    switch (this.type_of) {
      case 'date':
        return this.returnDateValue(date_string.value);

      case 'datetime':
        return this.returnDateTimeValue(date_string.value);

      default:
        return null;
    }
  }

  private returnDateValue(date: Date): number {
    switch (this.operator) {
      case 'HAS_DAY':
        return date.getUTCDate();

      case 'HAS_MONTH':
        return date.getUTCMonth() + 1;

      case 'HAS_YEAR':
        return date.getUTCFullYear();

      default:
        return date.getTime();
    }
  }

  private returnDateTimeValue(date: Date): number {
    return date.getTime();
  }
}

export default DateComparatorHelper;
