class DateString {
  public is_date: boolean;

  public value: Date = null;

  private received_value: string;

  private is_bar_date: boolean;

  private is_hyphen_date: boolean;

  private is_iso_date: boolean;

  constructor(value: string) {
    this.received_value = value;
    this.is_date_string();
    if (this.is_date) this.set_date_value();
  }

  private is_date_string() : void {
    const regex_bar_date = new RegExp('^\\d{2}\\/\\d{2}\\/\\d{4}$'); // dd/mm/yyyy
    this.is_bar_date = regex_bar_date.test(this.received_value);

    const regex_hyphen_date = new RegExp('^\\d{4}-\\d{2}-\\d{2}$'); // yyyy-mm-dd
    this.is_hyphen_date = regex_hyphen_date.test(this.received_value);

    this.is_iso_date = this.checkIsoDate();

    this.is_date = this.is_bar_date || this.is_hyphen_date || this.is_iso_date;
  }

  private checkIsoDate(): boolean {

    const regex_iso_date = new RegExp('^(\\d{4})-(\\d{2})-(\\d{2})T(\\d{2}):(\\d{2}):(\\d{2}).(\\d{3})Z$');

    const response = regex_iso_date.test(this.received_value);

    return response;
  }

  private set_date_value(): void {
    if (this.is_bar_date) {
      this.set_bar_date();
      return;
    }

    if (this.is_hyphen_date) {
      this.set_hyphen_date();
      return;
    }

    if (this.is_iso_date) {
      this.value = new Date(this.received_value);
    }
  }

  private set_bar_date(): void {
    const [day, month, year] = this.received_value.split('/');

    this.value = new Date([year, month, day].join('-'));
  }

  private set_hyphen_date(): void {
    this.value = new Date(this.received_value);
  }
}

export default DateString;
