import { expect } from 'chai';

import ICompare from '../src/interfaces/ICompare';
import IOperator from '../src/interfaces/IOperators';
import OperatorValidator from '../src/index';

describe('Test suite - Comparator BOOLEAN', () => {
  it('SUCCESS: Boolean with operator EQUALS (TRUE)', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'EQUALS';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });

  it('SUCCESS: Boolean with operator DIFF (TRUE)', () => {
    const trigger_value = true as const;
    const value_of_compared = false as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'DIFF';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });

  it('SUCCESS: Boolean with operator EQUALS (FALSE)', () => {
    const trigger_value = true as const;
    const value_of_compared = false as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'EQUALS';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('SUCCESS: Boolean with operator DIFF (FALSE)', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'DIFF';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('SUCCESS: Testing undefined OPERATOR', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const operator = undefined as any as IOperator;
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });

  it('SUCCESS: Testing DEFAULT case', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'DEFAULT' as IOperator;
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    // eslint-disable-next-line dot-notation
    const response = comparator['compare']();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with operator STARTING', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'STARTING';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with comparator ENDING', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'ENDING';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with comparator BIGGER', () => {
    const trigger_value = false as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'BIGGER';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with comparator MINOR', () => {
    const trigger_value = true as const;
    const value_of_compared = false as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'MINOR';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with comparator BIGGEREQ', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'BIGGEREQ';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with comparator MINOREQ', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'MINOREQ';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with comparator IN', () => {
    const trigger_value = [true] as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'IN';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with comparator HAS_DAY', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'HAS_DAY';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with comparator HAS_MONTH', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'HAS_MONTH';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: Boolean with comparator HAS_YEAR', () => {
    const trigger_value = true as const;
    const value_of_compared = true as const;
    const type_of_compared = typeof value_of_compared as ICompare;
    const operator = 'HAS_YEAR';
    const comparator = new OperatorValidator(type_of_compared, value_of_compared, trigger_value, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
});
