import { expect } from 'chai';

import OperatorValidator from '../src/index';
import ICompare from '../src/interfaces/ICompare';
import IOperator from '../src/interfaces/IOperators';

const bar_date_creator = (day:string, month:string, year:string) => [day, month, year].join('/');
const hyphen_date_creator = (year:string, month:string, day:string) => [year, month, day].join('-');

const day = '26' as const;
const month = '01' as const;
const year = '2022' as const;
const bar_string_date = bar_date_creator(day, month, year);
const hyphen_string_date = hyphen_date_creator(year, month, day);
const behavior_bar_date = bar_string_date;
const behavior_hyphen_date = hyphen_string_date;
const next_day = '27';
const next_month = '02';
const next_year = '2023';
const next_bar_string_date = bar_date_creator(next_day, next_month, next_year);
const next_hyphen_string_date = hyphen_date_creator(next_year, next_month, next_day);

function expect_compare(
  type_of_behavior_value: ICompare, behavior_value: unknown, event_value: unknown, operator: IOperator, expected = true
) {
  const comparator = new OperatorValidator(type_of_behavior_value, behavior_value, event_value, operator);
  expect(comparator.compare()).to.equal(expected);
}

describe('Test suite - Comparator DATE', () => {
  it('SUCCESS: Date with operator EQUALS | Bar Date', () => {
    const event_value = bar_string_date;
    const behavior_value = bar_string_date;
    const type_of_compared = typeof behavior_value as ICompare;
    const operator = 'EQUALS';
    expect_compare(type_of_compared, behavior_value, event_value, operator);
  });

  it('SUCCESS: Date with operator EQUALS | Hyphen Date', () => {
    const event_value = hyphen_string_date;
    const behavior_value = hyphen_string_date;
    const type_of_compared = typeof behavior_value as ICompare;
    const operator = 'EQUALS';
    expect_compare(type_of_compared, behavior_value, event_value, operator);
  });

  it('SUCCESS: Date with operator HAS_DAY', () => {
    const operator = 'HAS_DAY';

    // behavior_bar_string_date
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, hyphen_string_date, operator);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, bar_string_date, operator);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, Number(day), operator);
    // behavior_hyphen_string_date
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, hyphen_string_date, operator);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, bar_string_date, operator);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, Number(day), operator);
  });

  it('SUCCESS: Date with operator HAS_MONTH', () => {
    const operator = 'HAS_MONTH';

    // behavior_bar_string_date
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, hyphen_string_date, operator);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, bar_string_date, operator);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, Number(month), operator);
    // behavior_hyphen_string_date
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, hyphen_string_date, operator);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, bar_string_date, operator);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, Number(month), operator);
  });

  it('SUCCESS: Date with operator HAS_YEAR', () => {
    const operator = 'HAS_YEAR';

    // behavior_bar_string_date
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, hyphen_string_date, operator);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, bar_string_date, operator);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, Number(year), operator);
    // behavior_hyphen_string_date
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, hyphen_string_date, operator);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, bar_string_date, operator);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, Number(year), operator);
  });

  it('SUCCESS: Date with operator IN_WEEK', () => {
    const operator = 'IN_WEEK';

    // behavior_bar_string_date
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, '2022-01-24', operator);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, '24/01/2022', operator);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, Number(1642993200000), operator);
    // behavior_bar_string_date
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, '2022-01-23', operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, '23/01/2022', operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, Number(1642906800000), operator, false);
    // behavior_hyphen_string_date
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, '2022-01-24', operator);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, '24/01/2022', operator);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, Number(1642993200000), operator);
    // behavior_hyphen_string_date
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, '2022-01-23', operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, '23/01/2022', operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, Number(1642906800000), operator, false);
  })

  it('ERROR: Date with operator HAS_DAY', () => {
    const operator = 'HAS_DAY';

    // behavior_bar_string_date
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, next_hyphen_string_date, operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, next_bar_string_date, operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, Number(next_day), operator, false);
    // behavior_hyphen_string_date
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, next_hyphen_string_date, operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, next_bar_string_date, operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, Number(next_day), operator, false);
  });

  it('ERROR: Date with operator HAS_MONTH', () => {
    const operator = 'HAS_MONTH';

    // behavior_bar_string_date
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, next_hyphen_string_date, operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, next_bar_string_date, operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, Number(next_month), operator, false);
    // behavior_hyphen_string_date
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, next_hyphen_string_date, operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, next_bar_string_date, operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, Number(next_month), operator, false);
  });

  it('ERROR: Date with operator HAS_YEAR', () => {
    const operator = 'HAS_YEAR';

    // behavior_bar_string_date
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, next_hyphen_string_date, operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, next_bar_string_date, operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, Number(next_year), operator, false);
    // behavior_hyphen_string_date
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, next_hyphen_string_date, operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, next_bar_string_date, operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, Number(next_year), operator, false);
  });

  it('ERROR: Date with operator Default', () => {
    let operator;

    // behavior_bar_string_date
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, next_hyphen_string_date, operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, next_bar_string_date, operator, false);
    expect_compare(typeof behavior_bar_date as ICompare, behavior_bar_date, Number(next_year), operator, false);
    // behavior_hyphen_string_date
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, next_hyphen_string_date, operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, next_bar_string_date, operator, false);
    expect_compare(typeof behavior_hyphen_date as ICompare, behavior_hyphen_date, Number(next_year), operator, false);
  });
});
