import { expect } from 'chai';

import ICompare from '../src/interfaces/ICompare';
import IOperator from '../src/interfaces/IOperators';
import OperatorValidator from '../src/index';

const day = '26';
const month = '01';
const year = '2022';
const bigger_year = '2023';
const minor_year = '2021';
const bar_string_date = [day, month, year].join('/');
const hyphen_string_date = [year, month, day].join('-');

function expect_compare(
  type_of_behavior_value: ICompare, behavior_value: unknown, event_value: unknown, operator: IOperator, expected = true
) {
  const comparator = new OperatorValidator(type_of_behavior_value, behavior_value, event_value, operator);
  expect(comparator.compare()).to.equal(expected);
}

describe('Test suite - Comparator DATETIME', () => {
  it('SUCCESS: Datetime with operator BIGGER', () => {
    const operator = 'BIGGER';
    const behavior_value_bar = [day, month, bigger_year].join('/');
    const behavior_value_hyphen = [bigger_year, month, day].join('-');
    const type_behavior_bar = typeof behavior_value_bar as ICompare;
    const type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator);
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator);

    // behavior_hyphen_string_date
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator);
  });

  it('SUCCESS: Datetime with operator BIGGEREQ', () => {
    const operator = 'BIGGEREQ';
    let behavior_value_bar = bar_string_date;
    let behavior_value_hyphen = hyphen_string_date;
    let type_behavior_bar = typeof behavior_value_bar as ICompare;
    let type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date EQUAL
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator);

    // behavior_hyphen_string_date EQUAL
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator);

    behavior_value_bar = [day, month, bigger_year].join('/');
    behavior_value_hyphen = [bigger_year, month, day].join('-');
    type_behavior_bar = typeof behavior_value_bar as ICompare;
    type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date BIGGER
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator);

    // behavior_hyphen_string_date BIGGER
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator);
  });

  it('SUCCESS: Datetime with operator MINOR', () => {
    const operator = 'MINOR';
    const behavior_value_bar = [day, month, minor_year].join('/');
    const behavior_value_hyphen = [minor_year, month, day].join('-');
    const type_behavior_bar = typeof behavior_value_bar as ICompare;
    const type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator);

    // behavior_hyphen_string_date
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator);
  });

  it('SUCCESS: Datetime with operator MINOREQ', () => {
    const operator = 'MINOREQ';
    let behavior_value_bar = bar_string_date;
    let behavior_value_hyphen = hyphen_string_date;
    let type_behavior_bar = typeof behavior_value_bar as ICompare;
    let type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date EQUAL
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator);

    // behavior_hyphen_string_date EQUAL
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator);

    behavior_value_bar = [day, month, minor_year].join('/');
    behavior_value_hyphen = [minor_year, month, day].join('-');
    type_behavior_bar = typeof behavior_value_bar as ICompare;
    type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date MINOR
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator);

    // behavior_hyphen_string_date MINOR
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator);
  });

  it('ERROR: Datetime with operator BIGGER', () => {
    const operator = 'BIGGER';
    let behavior_value_bar = bar_string_date;
    let behavior_value_hyphen = hyphen_string_date;
    let type_behavior_bar = typeof behavior_value_bar as ICompare;
    let type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date EQUAL
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator, false);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator, false);

    // behavior_hyphen_string_date EQUAL
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator, false);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator, false);

    behavior_value_bar = [day, month, minor_year].join('/');
    behavior_value_hyphen = [minor_year, month, day].join('-');
    type_behavior_bar = typeof behavior_value_bar as ICompare;
    type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date MINOR
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator, false);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator, false);

    // behavior_hyphen_string_date MINOR
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator, false);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator, false);
  });

  it('ERROR: Datetime with operator BIGGEREQ', () => {
    const operator = 'BIGGEREQ';
    const behavior_value_bar = [day, month, minor_year].join('/');
    const behavior_value_hyphen = [minor_year, month, day].join('-');
    const type_behavior_bar = typeof behavior_value_bar as ICompare;
    const type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date MINOR
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator, false);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator, false);

    // behavior_hyphen_string_date MINOR
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator, false);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator, false);
  });

  it('ERROR: Datetime with operator MINOR', () => {
    const operator = 'MINOR';
    let behavior_value_bar = bar_string_date;
    let behavior_value_hyphen = hyphen_string_date;
    let type_behavior_bar = typeof behavior_value_bar as ICompare;
    let type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date EQUAL
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator, false);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator, false);

    // behavior_hyphen_string_date EQUAL
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator, false);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator, false);

    behavior_value_bar = [day, month, bigger_year].join('/');
    behavior_value_hyphen = [bigger_year, month, day].join('-');
    type_behavior_bar = typeof behavior_value_bar as ICompare;
    type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date BIGGER
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator, false);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator, false);

    // behavior_hyphen_string_date BIGGER
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator, false);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator, false);
  });

  it('ERROR: Datetime with operator MINOREQ', () => {
    const operator = 'MINOREQ';
    const behavior_value_bar = [day, month, bigger_year].join('/');
    const behavior_value_hyphen = [bigger_year, month, day].join('-');
    const type_behavior_bar = typeof behavior_value_bar as ICompare;
    const type_behavior_hyphen = typeof behavior_value_hyphen as ICompare;

    // behavior_bar_string_date BIGGER
    expect_compare(type_behavior_bar, behavior_value_bar, hyphen_string_date, operator, false);
    expect_compare(type_behavior_bar, behavior_value_bar, bar_string_date, operator, false);

    // behavior_hyphen_string_date BIGGER
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, hyphen_string_date, operator, false);
    expect_compare(type_behavior_hyphen, behavior_value_hyphen, bar_string_date, operator, false);
  });
});
