import { expect } from 'chai';

import ICompare from '../src/interfaces/ICompare';
import IOperator from '../src/interfaces/IOperators';
import OperatorValidator from '../src/index';

describe('Test suite - Comparator STRING', () => {
  it('SUCCESS: String with operator EQUALS (TRUE)', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'EQUALS';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });
  it('SUCCESS: String with operator DIFF (TRUE)', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'foca' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'DIFF';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });
  it('SUCCESS: String with operator STARTING (TRUE)', () => {
    const value_to_compare = 'ic' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'STARTING';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });
  it('SUCCESS: String with comparator ENDING (TRUE)', () => {
    const value_to_compare = 'bs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'ENDING';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });
  it('SUCCESS: String with comparator IN (TRUE)', () => {
    const value_to_compare = ['iclubs', 'not_iclubs'] as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'IN';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });
  it('SUCCESS: String with comparator NOT_IN (TRUE)', () => {
    const value_to_compare = ['iclubs'] as const;
    const value = 'foca' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'NOT_IN';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });
  it('SUCCESS: Testing undefined OPERATOR', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const operator = undefined as any as IOperator;
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(true);
  });
  it('SUCCESS: Testing DEFAULT case', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'DEFAULT' as IOperator;
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    // eslint-disable-next-line dot-notation
    const response = comparator['compare']();
    expect(response).to.equal(false);
  });
  it('SUCCESS: String with operator EQUALS (FALSE)', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'foca' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'EQUALS';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
  it('SUCCESS: String with operator DIFF (FALSE)', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'DIFF';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
  it('SUCCESS: String with operator STARTING (FALSE)', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'fo' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'STARTING';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
  it('SUCCESS: String with comparator ENDING (FALSE)', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'ca' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'ENDING';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
  it('SUCCESS: String with comparator IN (FALSE)', () => {
    const value_to_compare = ['iclubs'] as const;
    const value = 'foca' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'IN';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
  it('SUCCESS: String with comparator NOT_IN (FALSE)', () => {
    const value_to_compare = ['iclubs'] as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'NOT_IN';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
  it('ERROR: String with comparator BIGGER', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs_focas' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'BIGGER';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
  it('ERROR: String with comparator MINOR', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'ic' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'MINOR';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
  it('ERROR: String with comparator BIGGEREQ', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'BIGGEREQ';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
  it('ERROR: String with comparator MINOREQ', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'MINOREQ';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: String with comparator HAS_DAY', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'HAS_DAY';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: String with comparator HAS_MONTH', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'HAS_MONTH';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });

  it('ERROR: String with comparator HAS_YEAR', () => {
    const value_to_compare = 'iclubs' as const;
    const value = 'iclubs' as const;
    const type_of_value = typeof value as ICompare;
    const operator = 'HAS_YEAR';
    const comparator = new OperatorValidator(type_of_value, value, value_to_compare, operator);
    const response = comparator.compare();
    expect(response).to.equal(false);
  });
});
