import { expect } from "chai";
import { COREN_DAYS_VALUES } from './../src/constants/date';
import { days_in_milliseconds, getDateAtEndOfDay, getDateDaysAgo, getDateOnMidnight, getDayOfTheSameWeek, hours_in_milliseconds, isSameWeek, minutes_in_milliseconds, seconds_in_milliseconds } from './../src/utils/date';


const date = (d: string) => new Date(d);

describe('Test Suit - Date Utils', () => {
  it('seconds_in_milliseconds', () => {
    expect(seconds_in_milliseconds(1)).to.be.equal(1000);
  })
  it('minutes_in_milliseconds', () => {
    expect(minutes_in_milliseconds(1)).to.be.equal(60000);
  })
  it('hours_in_milliseconds', () => {
    expect(hours_in_milliseconds(1)).to.be.equal(3600000);
  })
  it('days_in_milliseconds', () => {
    expect(days_in_milliseconds(1)).to.be.equal(86400000);
  })
  it('getDateOnMidnight', () => {
    expect(getDateOnMidnight(date('2023-12-14T15:23:32.231Z'))).to.deep.equal(date('2023-12-14T00:00:00.000Z'));
  } )
  it('getDateAtEndOfDay', () => {
    expect(getDateAtEndOfDay(date('2023-12-14T15:23:32.231Z'))).to.deep.equal(date('2023-12-14T23:59:59.999Z'));
  } )
  it('getDateDaysAgo', () => {
    expect(getDateDaysAgo(date('2023-12-14T15:23:32.231Z'),1)).to.deep.equal(date('2023-12-13T00:00:00.000Z'));
  } )
  it('getDayOfTheSameWeek', () => {
    expect(getDayOfTheSameWeek(date('2023-12-14T15:23:32.231Z'), COREN_DAYS_VALUES.MONDAY)).to.deep.equal(date('2023-12-11T00:00:00.000Z'));
  } )
  it('isSameWeek', () => {
    expect(isSameWeek(date('2023-12-14T15:23:32.231Z'), date('2023-12-10T15:23:32.231Z'))).to.equal(false);
    expect(isSameWeek(date('2023-12-14T15:23:32.231Z'), date('2023-12-11T15:23:32.231Z'))).to.equal(true);
    expect(isSameWeek(date('2023-12-14T15:23:32.231Z'), date('2023-12-17T15:23:32.231Z'))).to.equal(true);
    expect(isSameWeek(date('2023-12-14T15:23:32.231Z'), date('2023-12-18T15:23:32.231Z'))).to.equal(false);
  } )
})