/* eslint-disable dot-notation */
import { expect } from 'chai';

import DateString from '../src/utils/date_string';

const day = '26';
const month = '10';
const year = '2022';
const time = 'T00:00:00.000Z';
const bar_string_date = [day, month, year].join('/');
const hyphen_string_date = [year, month, day].join('-');
const iso_string_date = [hyphen_string_date, time].join('');
const date_to_compare = new Date(hyphen_string_date);
const date_to_compare_iso = new Date(iso_string_date);

describe('Test suite - Date String', () => {
  it('SUCCESS: Set Date Value (Bar Date)', () => {
    const date_string = new DateString(bar_string_date);

    expect(date_string['is_bar_date']).to.equal(true);
    expect(date_string['is_hyphen_date']).to.equal(false);
    expect(date_string['is_iso_date']).to.equal(false);
    expect(date_string.is_date).to.equal(true);
    expect(date_string.value.toISOString()).to.equal(date_to_compare.toISOString());
  });

  it('SUCCESS: Set Date Value (Hyphen Date)', () => {
    const date_string = new DateString(hyphen_string_date);

    expect(date_string['is_bar_date']).to.equal(false);
    expect(date_string['is_hyphen_date']).to.equal(true);
    expect(date_string['is_iso_date']).to.equal(false);
    expect(date_string.is_date).to.equal(true);
    expect(date_string.value.toISOString()).to.equal(date_to_compare.toISOString());
  });

  it('SUCCESS: Set Date Value (ISO Date)', () => {
    const date_string = new DateString(iso_string_date);

    expect(date_string['is_bar_date']).to.equal(false);
    expect(date_string['is_hyphen_date']).to.equal(false);
    expect(date_string['is_iso_date']).to.equal(true);
    expect(date_string.is_date).to.equal(true);
    expect(date_string.value.toISOString()).to.equal(date_to_compare_iso.toISOString());
  });

  it('SUCCESS: Set Date Value (No Date)', () => {
    const date_string = new DateString('iclubs');

    expect(date_string['is_bar_date']).to.equal(false);
    expect(date_string['is_hyphen_date']).to.equal(false);
    expect(date_string.is_date).to.equal(false);
    date_string['set_date_value']();
    expect(date_string.value).to.equal(null);
  });
});
