"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ic_logs_1 = require("ic-logs");
const codeMessages_1 = tslib_1.__importDefault(require("./constants/codeMessages"));
const comparableAndOperators_1 = require("./constants/comparableAndOperators");
const exceptions_1 = tslib_1.__importDefault(require("./exceptions/exceptions"));
const date_1 = require("./utils/date");
const date_comparator_1 = tslib_1.__importDefault(require("./utils/date_comparator"));
const date_string_1 = tslib_1.__importDefault(require("./utils/date_string"));
class OperatorValidator {
    constructor(type_of_value, value, value_to_compare, operator) {
        this.type_of_value = type_of_value;
        this.value = value;
        this.value_to_compare = value_to_compare;
        this.operator = operator || 'EQUALS';
        this.comparable_types_and_operators = comparableAndOperators_1.COMPARABLE_TYPES_AND_OPERATORS;
    }
    compare() {
        try {
            this.verifyTypeAndOperator();
            const response = this.compareOperators();
            return response;
        }
        catch (error) {
            (0, ic_logs_1.errorLog)('[Compare]:', error.message);
            return false;
        }
    }
    setDateType(value_date) {
        if (this.comparable_types_and_operators.string.includes(this.operator))
            return;
        const date_comparator_helper = new date_comparator_1.default(this.operator, value_date.value, this.value_to_compare);
        this.type_of_value = date_comparator_helper.type_of;
        this.value = date_comparator_helper.primary_value_to_compare;
        this.value_to_compare = date_comparator_helper.secondary_value_to_compare;
    }
    verifyTypeAndOperator() {
        if (this.type_of_value === 'string') {
            const date_string = new date_string_1.default(this.value);
            if (date_string.is_date)
                this.setDateType(date_string);
        }
        const operators = this.comparable_types_and_operators[this.type_of_value];
        if (!operators || !operators.includes(this.operator)) {
            (0, ic_logs_1.errorLog)(`[INVALID_OPERATOR]: {"operator": "${this.operator}", "type": "${this.type_of_value}", "value": "${this.value}}`);
            throw new exceptions_1.default.BusinessError(codeMessages_1.default.INVALID_OPERATOR);
        }
    }
    compareOperators() {
        let array;
        let value_date;
        let value_to_compare_date;
        switch (this.operator) {
            case 'BIGGER':
                return this.value > this.value_to_compare;
            case 'BIGGEREQ':
                return this.value >= this.value_to_compare;
            case 'DIFF':
                return this.value !== this.value_to_compare;
            case 'ENDING':
                return `${this.value}`.endsWith(`${this.value_to_compare}`);
            case 'MINOR':
                return this.value < this.value_to_compare;
            case 'MINOREQ':
                return this.value <= this.value_to_compare;
            case 'STARTING':
                return `${this.value}`.startsWith(`${this.value_to_compare}`);
            case 'IN':
                array = this.value_to_compare;
                return array.includes(this.value);
            case 'NOT_IN':
                array = this.value_to_compare;
                debugger;
                return !array.includes(this.value);
            case 'HAS_DAY':
                value_date = this.value;
                return value_date.getUTCDate() === this.value_to_compare;
            case 'HAS_MONTH':
                value_date = this.value;
                return value_date.getUTCMonth() + 1 === this.value_to_compare;
            case 'HAS_YEAR':
                value_date = this.value;
                return value_date.getUTCFullYear() === this.value_to_compare;
            case 'IN_WEEK':
                value_date = this.value;
                value_to_compare_date = new Date(this.value_to_compare);
                return (0, date_1.isSameWeek)(value_to_compare_date, value_date);
            default:
                return this.value === this.value_to_compare;
        }
    }
}
exports.default = OperatorValidator;
