"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSameWeek = exports.getDayOfTheSameWeek = exports.getDateDaysAgo = exports.getDateAtEndOfDay = exports.getDateOnMidnight = exports.days_in_milliseconds = exports.hours_in_milliseconds = exports.minutes_in_milliseconds = exports.seconds_in_milliseconds = void 0;
const date_1 = require("../constants/date");
const seconds_in_milliseconds = (seconds) => seconds * 1000;
exports.seconds_in_milliseconds = seconds_in_milliseconds;
const minutes_in_milliseconds = (minutes) => (0, exports.seconds_in_milliseconds)(60 * minutes);
exports.minutes_in_milliseconds = minutes_in_milliseconds;
const hours_in_milliseconds = (hours) => (0, exports.minutes_in_milliseconds)(60 * hours);
exports.hours_in_milliseconds = hours_in_milliseconds;
const days_in_milliseconds = (days) => (0, exports.hours_in_milliseconds)(24 * days);
exports.days_in_milliseconds = days_in_milliseconds;
function getDateOnMidnight(date) {
    const midnight = new Date(date);
    midnight.setUTCHours(0, 0, 0, 0);
    return midnight;
}
exports.getDateOnMidnight = getDateOnMidnight;
function getDateAtEndOfDay(date) {
    const midnight = new Date(date);
    midnight.setUTCHours(23, 59, 59, 999);
    return midnight;
}
exports.getDateAtEndOfDay = getDateAtEndOfDay;
function getDateDaysAgo(date, days) {
    const timestamp = getDateOnMidnight(date).getTime();
    return new Date(timestamp - (0, exports.days_in_milliseconds)(days));
}
exports.getDateDaysAgo = getDateDaysAgo;
function getDayOfTheSameWeek(date, day) {
    const weekday = date_1.ISO_TO_COREN_DAYS_VALUES[date.getUTCDay()];
    if (day === weekday)
        return date;
    return getDateDaysAgo(date, weekday - day);
}
exports.getDayOfTheSameWeek = getDayOfTheSameWeek;
function isSameWeek(date, another_date) {
    const init = getDayOfTheSameWeek(date, date_1.COREN_DAYS_VALUES.MONDAY).getTime();
    const end = getDateAtEndOfDay(getDayOfTheSameWeek(date, date_1.COREN_DAYS_VALUES.SUNDAY)).getTime();
    const compare = another_date.getTime();
    return init <= compare && compare <= end;
}
exports.isSameWeek = isSameWeek;
