"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const codeMessages_1 = tslib_1.__importDefault(require("../constants/codeMessages"));
const comparableAndOperators_1 = require("../constants/comparableAndOperators");
const exceptions_1 = tslib_1.__importDefault(require("../exceptions/exceptions"));
const date_string_1 = tslib_1.__importDefault(require("./date_string"));
class DateComparatorHelper {
    constructor(operator, value, value_to_compare) {
        this.operator = operator;
        this.value = value;
        this.value_to_compare = value_to_compare;
        this.checkDateType();
    }
    checkDateType() {
        if (comparableAndOperators_1.COMPARABLE_TYPES_AND_OPERATORS.date.includes(this.operator)) {
            this.type_of = 'date';
            this.primary_value_to_compare = this.value;
            this.setJourneyValueToCompare();
            return this.type_of;
        }
        if (comparableAndOperators_1.COMPARABLE_TYPES_AND_OPERATORS.datetime.includes(this.operator)) {
            this.type_of = 'datetime';
            this.primary_value_to_compare = this.returnDateTimeValue(this.value);
            this.setJourneyValueToCompare();
        }
        return this.type_of;
    }
    setJourneyValueToCompare() {
        let value;
        switch (typeof this.value_to_compare) {
            case 'number':
                value = this.value_to_compare;
                break;
            case 'string':
                value = this.stringEventValueToCompare();
                break;
            default:
                break;
        }
        this.secondary_value_to_compare = value;
    }
    stringEventValueToCompare() {
        const date_string = new date_string_1.default(this.value_to_compare);
        if (!date_string.is_date)
            throw new exceptions_1.default.BusinessError(codeMessages_1.default.INVALID_TRIGGER_EVENT_VALUE);
        switch (this.type_of) {
            case 'date':
                return this.returnDateValue(date_string.value);
            case 'datetime':
                return this.returnDateTimeValue(date_string.value);
            default:
                return null;
        }
    }
    returnDateValue(date) {
        switch (this.operator) {
            case 'HAS_DAY':
                return date.getUTCDate();
            case 'HAS_MONTH':
                return date.getUTCMonth() + 1;
            case 'HAS_YEAR':
                return date.getUTCFullYear();
            default:
                return date.getTime();
        }
    }
    returnDateTimeValue(date) {
        return date.getTime();
    }
}
exports.default = DateComparatorHelper;
